import argparse
from threading import Thread

import shared_variables
from http_server import run_server
from utils import get_logger


def main():
    logger = get_logger("main")
    parser = argparse.ArgumentParser(
        description="Script to start backend Paging server."
    )
    parser.add_argument(
        "--port", type=int, default=8765, help="local port at which server listens."
    )
    args = parser.parse_args()
    try:
        shared_variables.initialize()
        logger.info("Shared variables initialization successful!!!")
    except Exception as e:
        logger.error(f"Shared variables initialization failed :{e}")
        return

    logger.info("Checking for file changes...")
    shared_variables.audio_processor.run()
    logger.info("Done!")

    listen_status_thread = Thread(target=shared_variables.pmaster.listen_status_message)
    listen_status_thread.daemon = True
    listen_status_thread.start()

    # NOTE: UNCOMMENT if the gateway should report its status as well.
    # master_status_thread = Thread(target=shared_variables.pmaster.send_status_message)
    # master_status_thread.daemon = True
    # master_status_thread.start()

    # Staring of the Server
    try:
        run_server(port=args.port)
    except Exception as e:
        logger.error(f"Paging server failed {e}")

    shared_variables.pmaster.exit()
    listen_status_thread.join()


if __name__ == "__main__":
    main()
