import json
from secrets import token_urlsafe
from utils import get_logger, read_json
import constants

logger = get_logger("Users")


class Users:
    ALL_USER = set([str(i) for i in range(1, 200)])

    def __init__(self, access=constants.ACCESS['user']):
        self._users = read_json("users_backup.json")
        self.occupied_user_id = set(self._users.keys())
        self.access = access  # TODO: Just an idea, not implemented. To be used with user access levels

    @property
    def new_secret(self):
        return token_urlsafe(16)

    @property
    def new_id(self):
        available_ids = Users.ALL_USER - self.occupied_user_id
        if len(available_ids) != 0:
            return min(available_ids)
        else:
            return None

    @property
    def bytes(self) -> bytes:
        return json.dumps(self._users).encode()

    def save(self):
        with open("users_backup.json", "w") as f:
            json.dump(self._users, f)

    # TODO: Just an idea, not implemented. To be used with user access levels
    def is_admin(self):
        return self.access == constants.ACCESS['admin']

    # TODO: Just an idea, not implemented. To be used with user access levels
    def allowed(self, access_level):
        return self.access >= access_level

    def add(self, user):
        user_name = user["name"]
        email = user["email"]
        new_id = self.new_id
        if new_id is not None:
            secret = self.new_secret
            self._users[new_id] = {"name": user_name, "email": email, "secret": secret}
            self.occupied_user_id.add(new_id)
            self.save()
            return True, {"id": new_id}
        else:
            return False, "Improper Request"

    def update(self, user):
        user_id = user["id"]
        user_name = user["name"]
        email = user["email"]
        if user_id in self._users:
            secret = self.new_secret
            self._users[user_id] = {"name": user_name, "email": email, "secret": secret}
            self.save()
            return True, "OK"
        else:
            logger.warning(f"User update unsuccessful!")
            return False, "User id doesn't exist!"

    def remove(self, user):
        user_ids = user["id"]
        for user_id in user_ids:
            if user_id in self._users and user_id != "0":
                del self._users[user_id]
                self.occupied_user_id.discard(user_id)
                logger.info(f"User {user_id} deletion sucessful!")
                self.save()
            if user_id == "0":
                logger.info(f"Admin user can't be deleted!")

            else:
                logger.warning(f"user id {user_id} is not present!")
        return True, "OK"

    def get_user_from_token(self, secret):
        for user_id in self._users.keys():
            if self._users[user_id]["secret"] == secret:
                return user_id

    def secret(self, user_id):
        return self._users[user_id]["secret"]

    def get_all_users(self) -> None:
        """
        Only use this function while debugging. DELETE later
        Returns:
        None
        """
        for user in self._users:
            logger.info("User: %s", user)
