from datetime import datetime

# import alsaaudio
import json
import socket
from subprocess import Popen, PIPE
import logging
import logging.handlers
import time
from inspect import currentframe, getframeinfo


def get_line_number():
    cf = currentframe()
    return cf.f_back.f_lineno


def get_active_devices(barp, listen_status_socket, pmaster, active_devices):
    while not pmaster.exit_flag:
        try:
            data, address = listen_status_socket.recvfrom(4096)
        except socket.timeout:
            continue
        msg = barp.decode(data)
        if msg[0]["message_type"] == 128:
            active_devices[msg[0]["sender_id"]] = {
                "state": msg[1]["state"],
                "time_stamp": datetime.now(),
            }

        # Removal of devices with stale status.
        current_time = datetime.now()
        for dev_id in list(active_devices.keys()):
            if (current_time - active_devices[dev_id]["time_stamp"]).seconds > 5:
                del active_devices[dev_id]
    print("Status listening stopped!")


def read_json(file_name):
    with open(file_name, "r") as f:
        return json.load(f)


def is_multicast(ip):
    """
    Performs check on ipv4 ip address to determin if it multicast or not.
    Note: it is assumed that the address is valid.
    Args:
        ip (str)
    Returns:
        (bool) : True if address is multicast and false if it isn't.
    """
    ip_address = [int(i) for i in ip.split(".")]
    if ip_address[0] >= 224 and ip_address[0] <= 239:
        return True
    else:
        return False


def get_uci(package, section, option):
    command = ["uci", "get", f"{package}.{section}.{option}"]
    output, error = Popen(command, stdout=PIPE, stderr=PIPE).communicate()
    if len(error) == 0:
        return output.decode().strip("\n")
    else:
        raise f"Unable to get {package}.{section}.{option} uci setting : {error}"


def uci_to_config_dict():
    config_dict = {}
    package = "icpgw"
    config_dict["ports"] = {
        "control": int(get_uci(package, "barp", "control_port")),
        "audio": int(get_uci(package, "barp", "audio_port")),
        "status": int(get_uci(package, "barp", "status_port")),
    }
    config_dict["audio_device"] = {
        "playback": get_uci(package, "server", "playback_device"),
        "record": "plug:analog_out",
    }
    config_dict["encoding_type"] = 0
    config_dict["multicast_group_address"] = get_uci(
        package, "barp", "audio_multicast_group_address"
    )
    config_dict["encoding_type"] = 0
    config_dict["paging_master_id"] = int(get_uci(package, "barp", "master_id"))
    config_dict["barp_ip"] = get_uci(package, "barp", "ip")
    config_dict["browser_timeout"] = int(get_uci(package, "server", "timeout"))
    config_dict["barp_header_kwargs"] = {
        "flags": int(get_uci(package, "barp", "flags")),
        "sender_id": int(get_uci(package, "barp", "master_id")),
        "receiver_id": 0,
        "sequence_number": int(get_uci(package, "barp", "master_id")),
    }
    config_dict["sample_rate"] = int(get_uci(package, "barp", "sample_rate"))
    config_dict["audio_format"] = get_uci(package, "barp", "format")
    # config_dict["input_device"] = get_uci(package, "audio", "input_device")  # TODO: Not tested
    # config_dict["period_size"] = int(get_uci(package, "audio", "period_size"))  # TODO: Not tested
    # config_dict["num_channels"] = int(get_uci(package, "audio", "num_channels"))  # TODO: Not tested
    # config_dict["output_device"] = get_uci(package, "audio", "output_device")  # TODO: Not tested
    return config_dict


def get_logger(name=""):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    ch = logging.FileHandler("/var/log/messages")
    ch.setLevel(logging.INFO)

    # create formatter
    formatter = logging.Formatter(  # Anton added threadName
        "%(asctime)s barix-ipam400 " + name + " [%(lineno)d] - %(levelname)s - %(message)s - [%(threadName)s]",
        datefmt="%b %d %H:%M:%S",
    )

    # add formatter to ch
    ch.setFormatter(formatter)

    # add ch to logger
    logger.addHandler(ch)
    return logger


def check_wav_file_format(file_name):
    tmp_file = file_name.split('.')

    if tmp_file[1] != "wav":
        return tmp_file[0]+".wav"
    else:
        return file_name
