let audioFilesList = {};
let sortedAudioFilesList = [];
let N_ROWS_audio = 4;
let N_COLS_audio = 3;
let audioPageIdx = 0;
let audioMaxPageIdx;
let audioOnClickCallback = null;
let submit_form = false;

function sortAudioFilesByName(filesReceived){
    let list = [];
    Object.keys(filesReceived).forEach(fileID => {
        list.push({
            "id": fileID,
            "name": filesReceived[fileID]["name"],
            "clients": filesReceived[fileID]["clients"]
        })
    });
    list.sort((a, b) => {
        let firstToCompare = "";
        if (a.name === ""){
            firstToCompare = ("Audio file "+a.id.toString()).toLowerCase();
        } else {
            firstToCompare = a.name.toLowerCase();
        }
        let secondToCompare = "";
        if (b.name === ""){
            secondToCompare = ("Audio file "+b.id.toString()).toLowerCase();
        } else {
            secondToCompare = b.name.toLowerCase();
        }
        return firstToCompare.localeCompare(secondToCompare)
    });
    return list;
}

function sortAudioFilesById(filesReceived){
    let list = [];
    Object.keys(filesReceived).forEach(audioFileID => {
        list.push({
            "id": audioFileID,
            "btn_name": filesReceived[audioFileID]["btn_name"],
            "file_name": filesReceived[audioFileID]["file_name"]
        })
    });
    list.sort((a, b) => {
        return parseInt(a.id) - parseInt(b.id);
    });
    return list;
}

function calculateMaxAudioPageIdx(){
    let max = Math.floor(sortedAudioFilesList.length/(N_ROWS_audio*N_COLS_audio)) - 1;
    let rest = sortedAudioFilesList.length % (N_ROWS_audio*N_COLS_audio)
    if (rest !== 0){
        max += 1;
    }
    audioMaxPageIdx = max;
}

/**
 Show modal with audio file details like name and clients list.
 Depending on the mode, it is possible to edit its contents.

 Mode: string -> "read-only" | "edit" | "new"
 Audio file name: string
**/

// Anton
// Removed allClientsSelected() function
// Removed buildClientCheckboxesTable(mode) 
// Removed checkClientCheckboxes(clientsList)

function openAudioFilesDetailsModal(audioFile, mode){
    buildModal("audioFile-modal");
    showPopup("audioFile-modal",false, false);

    const modalTitleElem = document.getElementById("audioFile-modal-title");
    const AudioFileBtnNameInputElem = document.getElementById("audioFile-name-input");
    const UploadAudioFileNameInputElem = document.getElementById("audioFile-upload-input");
   
    // audioFile name
    if (mode === "read-only"){
        //set modal title according to mode
        modalTitleElem.innerText = "Audio File Details";
        AudioFileBtnNameInputElem.value = AudioFileBtnNameInputElem.defaultValue = audioFile.btn_name;
        UploadAudioFileNameInputElem.value = UploadAudioFileNameInputElem.defaultValue = audioFile.file_name;
        AudioFileBtnNameInputElem.placeholder = "Audio file "+audioFile.id.toString();
    } else if (mode === "add") {
        //set modal title according to mode
        modalTitleElem.innerText = "New Audio File";
        AudioFileBtnNameInputElem.value = AudioFileBtnNameInputElem.defaultValue = "";
        UploadAudioFileNameInputElem.value = UploadAudioFileNameInputElem.defaultValue = "";
        UploadAudioFileNameInputElem.placeholder = "Upload a new audio file...";
        AudioFileBtnNameInputElem.placeholder = "Add message name";
    } else if (mode === "edit") {
        //set modal title according to mode
        modalTitleElem.innerText = "Edit Audio File";
        AudioFileBtnNameInputElem.value = AudioFileBtnNameInputElem.defaultValue = audioFile.btn_name;
        UploadAudioFileNameInputElem.value = UploadAudioFileNameInputElem.defaultValue = audioFile.file_name;
        UploadAudioFileNameInputElem.placeholder = "Current file: " + audioFile.file_name;
        AudioFileBtnNameInputElem.placeholder = "Audio File " + audioFile.id.toString();
    }

    if (window.screen.width < 500){
        AudioFileBtnNameInputElem.setAttribute("style", "font-size: 24px;");
        modalTitleElem.setAttribute("style", "font-size:40px;")
        const audioFileNameLabel = document.getElementById("audioFile-name-label");
        audioFileNameLabel.setAttribute("style", "font-size: 24px;");
        const audioFileUploadLabel = document.getElementById("audioFile-upload-label");
        audioFileUploadLabel.setAttribute("style", "font-size: 24px;");
    }

    // Anton removed checkboxes related code

    //delete audio file btn configuration
    const deleteAudioFileBtn = document.getElementById("deleteAudioFileBtn");
    if (mode === "edit") {
        deleteAudioFileBtn.hidden = false;
        deleteAudioFileBtn.onclick = function () {
            removeAudioFile(audioFile.id);
        }
        if (window.screen.width < 500){
            deleteAudioFileBtn.setAttribute("style", "font-size: 24px; margin-right: 100px; height: 40px;");
        }
    } else {
        deleteAudioFileBtn.hidden = true;
    }

    //save and cancel buttons configuration
    if (mode === "edit" || mode === "add"){
        const saveBtn = document.getElementById("modal-submit-btn");
        saveBtn.hidden = false;
        if (mode === "edit") {
            saveBtn.onclick = function () {
                saveAudioBtnOnClick(mode, audioFile.id);
            }
        } else {
            saveBtn.onclick = function () {
                saveAudioBtnOnClick(mode, null);
            }
        }
        if (window.screen.width < 500) {
            saveBtn.setAttribute("style", "font-size: 24px; height: 40px;");
        }
        const cancelBtn = document.getElementById("modal-cancel-btn");
        cancelBtn.hidden = false;
        if (window.screen.width < 500) {
            cancelBtn.setAttribute("style", "font-size: 24px; height: 40px;");
        }
    }
}

// Anton
// Removed selectAllCheckboxOnClick()

function showAudioFilesDetails(audioFile) {
    openAudioFilesDetailsModal(audioFile, "read-only");
}

function editAudioFilesDetails(audioFile) {
    openAudioFilesDetailsModal(audioFile, "edit");
}

function newAudioFile() {
    openAudioFilesDetailsModal(null, "add");
}

function removeAudioFile(audioFileID) {
    let audioFileName = audioFileList[audioFileID]["btn_name"];  // was name before
    if (audioFileName === ""){
        audioFileName = "Audio File "+audioFileID.toString();
    }

    let r = confirm("Are you sure you wan't to remove \""+audioFileName+"\"?");
    if (r === true) {
        //OK
        // close modal
        closePopup("audioFile-modal");
        // hide audio files div
        commonToolsHideElemId('main-content');
        // show spinner
        commonToolsShowElemId('loading-spinner');

        // wait for the user to see the loading spinner
        setTimeout(function (){
            removeAudioFileRequest(audioFileID, audioFileName);
        }, 1500);
    } else {
    }
}

function handleHTTPSuccessResponseAudio(msg) {

    // hide audio files div
    commonToolsHideElemId('loading-spinner');
    // show spinner
    commonToolsShowElemId('main-content');
    // reload data
    createAudioFilesList();
    const audioFilesDiv = document.getElementById('audioFile-div'); // audioFiles-section
    if (audioFilesDiv) {
        audioFilesDiv.innerHTML = "";
        const mediaInternal = renderAudioFileDiv();
        audioFilesDiv.appendChild(mediaInternal);
        if (sortedAudioFilesList.length !== 0) renderAudioFilesTable();
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
    // show notification
    notify.notify({message: msg, level:'good'});
}


function handleHTTPFailResponseAudio(msg, status, error) {
    // hide audio files div
    commonToolsHideElemId('loading-spinner');
    // show spinner
    commonToolsShowElemId('main-content');
    // reload data
    createAudioFilesList();

    const audioFilesDiv = document.getElementById('audioFile-div'); // audioFiles-section
    if (audioFilesDiv) {
        audioFilesDiv.innerHTML = "";
        const mediaInternal = renderAudioFileDiv();
        audioFilesDiv.appendChild(mediaInternal);
        if (sortedAudioFilesList.length !== 0) renderAudioFilesTable();
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }

    let message = msg;
    message = message + ". Error: " + error;
    // show notification
    notify.notify({message: message, level:'danger'});
}

function submitChangesAudio(mode, audioFileID, audioFileName, uploadFileName) {
    // close modal
    closePopup("audioFile-modal");
    // hide audio files div
    commonToolsHideElemId('main-content');
    // show spinner
    commonToolsShowElemId('loading-spinner');

    // wait for the user to see the loading spinner
    setTimeout(function (){
        // make request
        if (mode === "add") {             
            document.forms["audioFile_form"].submit();  // Physical upload of the file                
            addAudioFileRequest(audioFileName, uploadFileName);  //Storing the file data in the json file
        } else if (mode === "edit"){
            if (submit_form == true){                
                document.forms["audioFile_form"].submit();  // Physical upload of the file
                submit_form = false; // Set immediately back to false so no files are uploaded while Editting
            }
            editAudioFileRequest(audioFileID, audioFileName, uploadFileName); //Storing the file data in the json file
        }
    }, 1500);
}

function saveAudioBtnOnClick(mode, audioFileID) {
    let audioFileName = document.getElementById("audioFile-name-input").value;
    let uploadedFileName = document.getElementById("audioFile-upload-input").value;

    //check if name is valid
    if (audioFileName.replace(/\s+/g, '') === ""){
        audioFileName = "";
    }

    //check if file name is valid
    if (uploadedFileName.replace(/\s+/g, '') === ""){
        uploadedFileName = "";
    }

    submitChangesAudio(mode, audioFileID, audioFileName, uploadedFileName);
}

function clearAudioFilesTable() {
    const mediaTable = document.getElementById("audioFiles-table");
    mediaTable.innerHTML="";
}

function renderAudioFilesTable(){

    clearAudioFilesTable();

    const mediaTable = document.getElementById("audioFiles-table");
    mediaTable.setAttribute("cellspacing", "0");
    mediaTable.setAttribute("cellpadding", "0");
    mediaTable.setAttribute("class", "labelValue");

    for (let i = 0; i < N_ROWS_audio; i++){
        let stop = false;
        const tableRow = mediaTable.insertRow(-1);
        for (let j = 0; j < N_COLS_audio; j++){
            if ((i * N_COLS_audio + j + audioPageIdx * N_ROWS_audio * N_COLS_audio) < sortedAudioFilesList.length){
                let audioFile = sortedAudioFilesList[i * N_COLS_audio + j + audioPageIdx * N_ROWS_audio * N_COLS_audio];
                const tableCell = tableRow.insertCell(-1);
                const audioFileBtn = document.createElement("button");
                audioFileBtn.setAttribute("class", "audioFile-button");
                audioFileBtn.setAttribute("id", "audioFile-button-"+audioFile.id.toString());
                audioFileBtn.innerText = audioFile.btn_name;
                if (audioFile.btn_name === ""){
                    audioFileBtn.innerText = "Audio File "+audioFile.id.toString();
                }
                // show more
                audioFileBtn.onclick = function () {
                    audioOnClickCallback(audioFile);
                    // audioOnClickCallback(audioFile.file_name);
                }
                tableCell.appendChild(audioFileBtn);
            } else {
                stop = true;
                break;
            }
        }
        if (stop) break;
    }
}

function renderAudioFileDiv() {

    if (sortedAudioFilesList.length === 0) {
        const noFilesDivElem = document.createElement("div");
        noFilesDivElem.setAttribute("class", "section2");
        noFilesDivElem.setAttribute("style", "font-size: 13px; font-style: italic; padding: 4px; color: #888888; text-align: center; margin-top: 10%;");
        noFilesDivElem.innerText = "No audio files configured";
        return noFilesDivElem;
    }

    // create table div with audio files
    const mediaTable = document.createElement("table");
    mediaTable.setAttribute("id", "audioFiles-table");

    const mainTable = document.createElement("table");
    const tableRow = mainTable.insertRow(-1);

    tableRow.setAttribute("align", "center");
    tableRow.setAttribute("border", "0");
    tableRow.setAttribute("cellspacing", "0");
    tableRow.setAttribute("cellpadding", "0");

    const tableLeftCell = tableRow.insertCell(-1);
    // tableLeftCell.appendChild(arrowLeftDivElem);
    const tableMiddleCell = tableRow.insertCell(-1);
    tableMiddleCell.appendChild(mediaTable);
    const tableRightCell = tableRow.insertCell(-1);
    // tableRightCell.appendChild(arrowRightDivElem);

    return mainTable;

}

/**
 * Render the existing audio files
 */
function createAudioFilesList() {
    audioFileList = getListOfAudioFiles();
    //console.log(audioFileList);
    //sortedAudioFilesList = sortAudioFilesByName(audioFileList);
    sortedAudioFilesList = sortAudioFilesById(audioFileList);
    //console.log(sortedAudioFilesList);
    calculateMaxAudioPageIdx();
}

function loadAudioFiles(btnClickFunction) {
    audioOnClickCallback = btnClickFunction;
    if (window.screen.width < 500){
        const newAudioFileBtn = document.getElementById("newAudioFileBtn");
        if (newAudioFileBtn) newAudioFileBtn.setAttribute("style", "width: 170px; height: 60px; font-size: 24px;")
    }
    const audioFileDiv = document.getElementById('audioFile-div');  //audioFiles-section
    if (audioFileDiv) {
        createAudioFilesList();
        const mediaInternal = renderAudioFileDiv();
        audioFileDiv.appendChild(mediaInternal);
        if (sortedAudioFilesList.length !== 0) renderAudioFilesTable();
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
}

function setFileName(){
    var inputEl = document.getElementById("filename");
    var destinationFileName = document.getElementById("audioFile-upload-input");
    var fileName = inputEl.value;
    // Remove spaces in the name
    fileName = fileName.replace(/\s/g, '');
    // Remove path, keep file name only 
    fileName =  fileName.split('\\').pop().split('/').pop();
    // Get only the 7 first chars
    // fileName = fileName.substring(0,7);
    // Save the extension
    var ext = fileName.substr(fileName.lastIndexOf('.') + 1);
    // Save everything before the extensioin
    fileName = fileName.substring(0, fileName.lastIndexOf('.'));
    // Remove dots from the fileName because they are confusing
    fileName = fileName.replace(/\./g, '');
    // Add a prefix so that I can identify and delete all icons
    fileName = fileName + "." + ext;
    // Set icon file name in a DOM element to be used as
    // POST argument that will be read by the CGI script
    destinationFileName.value = fileName;
    submit_form = true; // Used to only upload files to the device when needed
}